@@ -69,7 +69,7 @@ int pthread_kill(pthread_t thread, int signo)
 }
 
 /* User-provided signal handlers */
-typedef void (*arch_sighandler_t) __PMT ((int, SIGCONTEXT));
+typedef void (*arch_sighandler_t) (int, SIGCONTEXT);
 static union
 {
   arch_sighandler_t old;
@@ -79,8 +79,9 @@ static union
 /* The wrapper around user-provided signal handlers */
 static void pthread_sighandler(int signo, SIGCONTEXT ctx)
 {
-  pthread_descr self = thread_self();
+  pthread_descr self;
   char * in_sighandler;
+  self = thread_self();
   /* If we're in a sigwait operation, just record the signal received
      and return without calling the user's handler */
   if (THREAD_GETMEM(self, p_sigwaiting)) {
@@ -102,8 +103,9 @@ static void pthread_sighandler(int signo, SIGCONTEXT ctx)
 static void pthread_sighandler_rt(int signo, struct siginfo *si,
 				  struct ucontext *uc)
 {
-  pthread_descr self = thread_self();
+  pthread_descr self;
   char * in_sighandler;
+  self =  thread_self();
   /* If we're in a sigwait operation, just record the signal received
      and return without calling the user's handler */
   if (THREAD_GETMEM(self, p_sigwaiting)) {
@@ -123,7 +125,7 @@ static void pthread_sighandler_rt(int signo, struct siginfo *si,
 
 /* The wrapper around sigaction.  Install our own signal handler
    around the signal. */
-int sigaction(int sig, const struct sigaction * act,
+int __sigaction(int sig, const struct sigaction * act,
               struct sigaction * oact)
 {
   struct sigaction newact;
@@ -132,7 +134,10 @@ int sigaction(int sig, const struct sigaction * act,
   if (sig == __pthread_sig_restart ||
       sig == __pthread_sig_cancel ||
       (sig == __pthread_sig_debug && __pthread_sig_debug > 0))
-    return EINVAL;
+    {
+      __set_errno (EINVAL);
+      return -1;
+    }
   if (act)
     {
       newact = *act;
@@ -148,7 +153,7 @@ int sigaction(int sig, const struct sigaction * act,
     }
   else
     newactp = NULL;
-  if (__sigaction(sig, newactp, oact) == -1)
+  if (__libc_sigaction(sig, newactp, oact) == -1)
     return -1;
   if (sig > 0 && sig < NSIG)
     {
@@ -161,6 +166,7 @@ int sigaction(int sig, const struct sigaction * act,
     }
   return 0;
 }
+strong_alias(__sigaction, sigaction)
 
 /* A signal handler that does nothing */
 static void pthread_null_sighandler(int sig) { }
@@ -189,8 +195,8 @@ int sigwait(const sigset_t * set, int * sig)
         s != __pthread_sig_debug) {
       sigdelset(&mask, s);
       if (sighandler[s].old == NULL ||
-	  sighandler[s].old == (arch_sighandler_t) SIG_DFL ||
-	  sighandler[s].old == (arch_sighandler_t) SIG_IGN) {
+          sighandler[s].old == (arch_sighandler_t) SIG_DFL ||
+          sighandler[s].old == (arch_sighandler_t) SIG_IGN) {
         sa.sa_handler = pthread_null_sighandler;
         sigemptyset(&sa.sa_mask);
         sa.sa_flags = 0;
